package org.foxarmy.barcodescannerforemployees

import android.content.Context
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import okhttp3.*
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.RequestBody.Companion.asRequestBody
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Category
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import java.io.File
import java.io.FileOutputStream
import java.io.IOException
import java.util.concurrent.TimeUnit

class Net {
    var language = "en-US"
    var server = "bsfe.foxarmy.org"
    var token = ""

    fun serverIsAvailable(context: Context, callback: (Boolean) -> Unit) {
        if (!isInternetConnectionAvailable(context)) {
            callback(false)
        }
        CoroutineScope(Dispatchers.IO).launch {
            var flag = false
            val client = OkHttpClient();
            val request = Request.Builder()
                .url("https://$server/status")
                .get()
                .build();

            try {
                val response = client.newCall(request).execute();
                flag = response.code == 200;
            } catch (e: Exception) {
                flag = false;
            }
            callback(flag)
        }
    }

    fun requestProductFromOnlineDB(barcode: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response
            val url = "https://ean-online.ru/match.php"

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("barcode", barcode)
                .build()

            val request = Request.Builder()
                .url(url)
                .post(body)
                .addHeader("referer", "https://ean-online.ru")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun registerAccount(username: String, password: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("username", username)
                .add("password", password)
                .build()

            val request = Request.Builder()
                .url("https://$server/api/user/register")
                .post(body)
                .addHeader("content-type", "application/x-www-form-urlencoded")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun login(username: String, password: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response
            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("username", username)
                .add("password", password)
                .build()
            val requestLogin = Request.Builder()
                .url("https://$server/api/user/login")
                .post(body)
                .addHeader("content-type", "application/x-www-form-urlencoded")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(requestLogin).execute()
            callback(response)
        }
    }

    fun uploadAbstractProduct(
        groupId: Int,
        abstractProduct: AbstractProduct,
        imageFile: File,
        callback: (Response) -> Unit,
    ) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = MultipartBody.Builder()
                .setType("multipart/form-data".toMediaType())
                .addFormDataPart("file", imageFile.name, imageFile.asRequestBody("image/png".toMediaTypeOrNull()))
                .addFormDataPart("groupId", groupId.toString())
                .addFormDataPart("localId", abstractProduct.id.toString())
                .addFormDataPart("barcode", abstractProduct.barcode)
                .addFormDataPart("name", abstractProduct.name)
                .addFormDataPart("net_weight", abstractProduct.netWeight.toString())
                .addFormDataPart("category", abstractProduct.category.toString())
                .addFormDataPart("unit", abstractProduct.unit.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/abstractproduct/create")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun changeUsername(newUsername: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
            body.add("username", newUsername)

            val requestBody = body.build()

            val request = Request.Builder()
                .url("https://$server/api/user/changeUsername")
                .post(requestBody)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun changePassword(newPassword: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
            body.add("password", newPassword)

            val requestBody = body.build()

            val request = Request.Builder()
                .url("https://$server/api/user/changePassword")
                .post(requestBody)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun createGroup(name: String, password: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            val client = OkHttpClient()

            val body = FormBody.Builder()

            val requestBody = body.build()

            val request = Request.Builder()
                .url("https://$server/api/group/create/$name")
                .post(requestBody)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            val response = client.newCall(request).execute()

            changeGroupPassword(name, password, { responseFromPassword ->
                callback(response)
            })

        }
    }

    fun joinGroup(id: Int, password: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("password", password)
                .build()

            val request = Request.Builder()
                .url("https://$server/api/group/join/$id")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun changeGroupPassword(name: String, password: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            getGroupId(name, { groupId ->
                val client = OkHttpClient()

                val body = FormBody.Builder()
                body.add("password", password)

                val requestBody = body.build()

                val request = Request.Builder()
                    .url("https://$server/api/group/password/$groupId")
                    .post(requestBody)
                    .addHeader("Authorization", "Bearer $token")
                    .addHeader("accept-language", language)
                    .build()
                response = client.newCall(request).execute()
                callback(response)
            });
        }
    }

    fun getGroupId(name: String, callback: (String) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/byName/$name")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
            callback(response.body!!.string())
        }
    }

    fun getGroupName(id: Int, callback: (String) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/byId/$id")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()
            response = client.newCall(request).execute()
            callback(response.body!!.string())
        }
    }

    fun getUsersInGroup(groupId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/getUsers/$groupId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun getMyGroups(callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/user/myGroups")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun getUsernameById(userId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/user/byId/$userId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun getGroupAdminId(groupId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/adminId/$groupId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun updateAbstractProduct(
        groupId: Int,
        abstractProduct: AbstractProduct,
        imageFile: File,
        callback: (Response) -> Unit,
    ) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = MultipartBody.Builder()
                .setType("multipart/form-data".toMediaType())
                .addFormDataPart("file", imageFile.name, imageFile.asRequestBody("image/png".toMediaTypeOrNull()))
                .addFormDataPart("groupId", groupId.toString())
                .addFormDataPart("localId", abstractProduct.id.toString())
                .addFormDataPart("barcode", abstractProduct.barcode)
                .addFormDataPart("name", abstractProduct.name)
                .addFormDataPart("net_weight", abstractProduct.netWeight.toString())
                .addFormDataPart("category", abstractProduct.category.toString())
                .addFormDataPart("unit", abstractProduct.unit.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/abstractproduct/update")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun uploadCategory(groupId: Int, category: Category, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("localId", category.id.toString())
                .add("categoryName", category.name)
                .add("groupId", groupId.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/category/create")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun updateCategory(groupId: Int, category: Category, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("localId", category.id.toString())
                .add("categoryName", category.name)
                .add("groupId", groupId.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/category/update")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun uploadProduct(groupId: Int, product: Product, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("localId", product.id.toString())
                .add("groupId", groupId.toString())
                .add("abstract_product_id", product.abstractProductId.toString())
                .add("amount", product.amount.toString())
                .add("date_of_production", product.dateOfProduction.toString())
                .add("expiry_date", product.dateOfExpiry.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/product/create")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun updateProduct(groupId: Int, product: Product, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("localId", product.id.toString())
                .add("groupId", groupId.toString())
                .add("abstract_product_id", product.abstractProductId.toString())
                .add("amount", product.amount.toString())
                .add("date_of_production", product.dateOfProduction.toString())
                .add("expiry_date", product.dateOfExpiry.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/product/update")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun synchronize(groupId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/user/synchronize/$groupId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun downloadImage(url: String, file: File, callback: () -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            val client = OkHttpClient.Builder()
                .connectTimeout(30, TimeUnit.SECONDS)
                .readTimeout(10, TimeUnit.MINUTES)
                .build()

            val request = Request.Builder()
                .url(url)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            val response = client.newCall(request).execute()
            if (!response.isSuccessful) throw IOException("Unexpected code $response")

            val fos = FileOutputStream(file)

            response.body?.byteStream()?.use { inputStream ->
                fos.use {
                    inputStream.copyTo(fos)
                }
            }
            callback()
        }
    }

    fun deleteCategory(groupId: Int, localId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/category/$groupId/$localId")
                .delete()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun deleteAbstractProduct(groupId: Int, localId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/abstractproduct/$groupId/$localId")
                .delete()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun deleteProduct(groupId: Int, localId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/product/$groupId/$localId")
                .delete()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun renameGroup(groupId: Int, newName: String, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("name", newName)
                .build()

            val request = Request.Builder()
                .url("https://$server/api/group/rename/$groupId")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun transfer_ownership(groupId: Int, userId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val body = FormBody.Builder()
                .add("userId", userId.toString())
                .build()

            val request = Request.Builder()
                .url("https://$server/api/group/transferOwnership/$groupId")
                .post(body)
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }

    fun leaveGroup(groupId: Int, callback: (Response) -> Unit) {
        CoroutineScope(Dispatchers.IO).launch {
            lateinit var response: Response

            val client = OkHttpClient()

            val request = Request.Builder()
                .url("https://$server/api/group/leave/$groupId")
                .get()
                .addHeader("Authorization", "Bearer $token")
                .addHeader("accept-language", language)
                .build()

            response = client.newCall(request).execute()
            callback(response)
        }
    }
}