package org.foxarmy.barcodescannerforemployees

import android.app.Activity
import android.content.Context
import android.content.ContextWrapper
import android.content.Intent
import android.content.SharedPreferences
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Matrix
import android.media.ExifInterface
import android.net.ConnectivityManager
import android.net.NetworkCapabilities
import android.net.Uri
import android.os.Build
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AlertDialog
import androidx.core.content.FileProvider
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import com.google.firebase.components.BuildConfig
import org.foxarmy.barcodescannerforemployees.activities.LoginActivity
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.security.MessageDigest
import java.text.SimpleDateFormat
import java.util.*
import kotlin.system.exitProcess

fun convertToUnixEpochTimestamp(dateString: String): Long {
    val format = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'")
    format.timeZone = TimeZone.getTimeZone("UTC")
    val date = format.parse(dateString)
    return date!!.time / 1000
}

fun getImageUri(activity: Activity, imageFile: File): Uri? {
    return FileProvider.getUriForFile(
        activity,
        BuildConfig.APPLICATION_ID + "." + activity.localClassName + ".provider",
        imageFile
    )
}

@RequiresApi(Build.VERSION_CODES.R)
fun generateThumbnailForImage(context: Context, imageHash: String) {
    val thumbnailsDir = File(context.cacheDir, "thumbnails")
    thumbnailsDir.mkdirs()
    val picturesDir = File(context.filesDir, "pictures")
    picturesDir.mkdirs()
    val thumbnailFile = File(thumbnailsDir, "$imageHash.webp")
    if (thumbnailFile.exists()) return

    val imageFile = File(picturesDir, "$imageHash.png")
    val imageContent = imageFile.inputStream().readBytes()
    var img = BitmapFactory.decodeByteArray(imageContent, 0, imageContent.size)

    val exif = ExifInterface(imageFile.absoluteFile.toString())
    val orientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL)
    val matrix = Matrix()

    when (orientation) {
        ExifInterface.ORIENTATION_ROTATE_90 -> matrix.postRotate(90F)
        ExifInterface.ORIENTATION_ROTATE_180 -> matrix.postRotate(180F)
        ExifInterface.ORIENTATION_ROTATE_270 -> matrix.postRotate(270F)
    }
    val rotated = Bitmap.createBitmap(img, 0, 0, img.width, img.height, matrix, true)
    val scaled = Bitmap.createScaledBitmap(rotated, rotated.width / 4, rotated.height / 4, true)
    scaled.compress(Bitmap.CompressFormat.WEBP_LOSSY, 25, FileOutputStream(thumbnailFile))
}

@OptIn(ExperimentalStdlibApi::class)
fun String.md5(): String {
    val md = MessageDigest.getInstance("MD5")
    val digest = md.digest(this.toByteArray())
    return digest.toHexString()
}

fun stripNetWeight(netWeight: String): Double {
    return removeSubstringsFromString(
        netWeight,
        arrayOf("Л", "л", "мл", "Мл", "г", "Г", "кг", "Кг", "шт", "Шт", ",", " ", ".")
    ).toDouble()
}

fun removeSubstringsFromString(text: String, toRemove: Array<String>): String {
    var result = text
    for (candidate in toRemove.iterator()) {
        result = result.replace(candidate, "")
    }
    return result
}

fun getActivity(context: Context?): Activity? {
    if (context == null) {
        return null
    } else if (context is ContextWrapper) {
        return if (context is Activity) {
            context
        } else {
            getActivity(context.baseContext)
        }
    }

    return null
}

@RequiresApi(Build.VERSION_CODES.O)
fun calculateProductFreshness(dateOfProduction: Long, dateOfExpiry: Long): Double {

    val productLifeSpan = dateOfExpiry - dateOfProduction;
    val lifeSpanLeft: Long = dateOfExpiry - Date().time / 1000

    return lifeSpanLeft / productLifeSpan.toDouble()
}

fun getUnitNameById(context: Context, id: Int): String {
    return when (id) {
        0 -> { context.getString(R.string.kilogram) }

        1 -> { context.getString(R.string.gram) }

        2 -> { context.getString(R.string.liter) }

        3 -> { context.getString(R.string.milliliter) }

        4 -> { context.getString(R.string.pieces) }

        else -> { "" }
    }
}

fun parseIntArray(input: String): IntArray {
    return input.trim('[', ']').split(",").map { it.trim().toInt() }.toIntArray()
}

fun parseStringList(input: String): List<String> {
    return input.trim('[', ']').split(",").map { it.trim() }.toList()
}

fun calculateMd5Hash(file: File): String {
    val digest = MessageDigest.getInstance("MD5")
    val fis = FileInputStream(file)
    val buffer = ByteArray(1024)
    var bytesRead = fis.read(buffer)
    while (bytesRead != -1) {
        digest.update(buffer, 0, bytesRead)
        bytesRead = fis.read(buffer)
    }
    fis.close()
    return bytesToHex(digest.digest())
}

fun bytesToHex(bytes: ByteArray): String {
    val hexString = StringBuilder()
    for (byte in bytes) {
        val hex = Integer.toHexString(0xff and byte.toInt())
        if (hex.length == 1) {
            hexString.append('0')
        }
        hexString.append(hex)
    }
    return hexString.toString()
}

fun isInternetConnectionAvailable(context: Context): Boolean {
    if (context.getSystemService(Context.CONNECTIVITY_SERVICE) == null) return false
    val connectivityManager =
        context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
    if (connectivityManager != null) {
        val capabilities =
            connectivityManager.getNetworkCapabilities(connectivityManager.activeNetwork)
        if (capabilities != null) {
            if (capabilities.hasTransport(NetworkCapabilities.TRANSPORT_CELLULAR)) {
                return true
            } else if (capabilities.hasTransport(NetworkCapabilities.TRANSPORT_WIFI)) {
                return true
            } else if (capabilities.hasTransport(NetworkCapabilities.TRANSPORT_ETHERNET)) {
                return true
            }
        }
    }
    return false
}

fun noInternetConnectionAvailableNotification(context: Context) {
    (context as Activity).runOnUiThread {
        AlertDialog.Builder(context)
            .setMessage(context.getString(R.string.no_internet_connection))
            .setCancelable(false)
            .setPositiveButton(R.string.quit) { _, _ ->
                exitProcess(0)
            }
            .setNeutralButton(R.string.logout) { _, _ ->
                val sharedPreferences = getPreferences(context)

                sharedPreferences.edit().putString("token", "").apply()
                sharedPreferences.edit().putString("server", "").apply()
                sharedPreferences.edit().putStringSet("groups", emptySet()).apply()
                sharedPreferences.edit().putString("currentGroup", "").apply()
                val intent = Intent(context, LoginActivity::class.java)
                context.startActivity(intent)
                context.finish()
            }.show()
    }
}

fun getPreferences(context: Context): SharedPreferences {
    return EncryptedSharedPreferences.create(
        "sensitive",
        MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
        context,
        EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
        EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
    )
}