package org.foxarmy.barcodescannerforemployees.activities

import android.app.ProgressDialog
import android.content.DialogInterface
import android.content.Intent
import android.content.SharedPreferences
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Matrix
import android.media.ExifInterface
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.widget.*
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.core.widget.addTextChangedListener
import com.journeyapps.barcodescanner.ScanContract
import com.journeyapps.barcodescanner.ScanIntentResult
import com.journeyapps.barcodescanner.ScanOptions
import okhttp3.Response
import org.foxarmy.barcodescannerforemployees.*
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import java.io.File
import java.io.FileOutputStream
import java.nio.file.Files
import java.nio.file.StandardCopyOption

class AddAbstractProductActivity : AppCompatActivity() {
    private lateinit var imageView: ImageView

    private lateinit var saveButton: Button
    private lateinit var takePictureButton: Button
    private lateinit var scanButton: Button

    private lateinit var barcodeText: EditText
    private lateinit var noBarcodeCheckBox: CheckBox
    private lateinit var productNameText: TextView
    private lateinit var netWeightText: TextView
    private lateinit var unitTypeSpinner: Spinner

    private lateinit var categorySpinner: Spinner

    private var abstractProduct: AbstractProduct? = null
    private lateinit var pictureFile: File
    private lateinit var picturesPath: File
    private var barcode: String = ""
    private var action: String = "new"

    private var scanningBarcode = false

    private lateinit var DAO: AbstractProductDAO
    private lateinit var sharedPreferences: SharedPreferences

    private lateinit var loadingDialog: ProgressDialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.fragment_add_abstract_product)

        loadingDialog = ProgressDialog(this)
        loadingDialog.setMessage(getString(R.string.loading_please_wait))
        loadingDialog.setCancelable(false)
        loadingDialog.setTitle(getString(R.string.loading))

        sharedPreferences = getPreferences(this)

        val dbHelper = DBStorageController(this, sharedPreferences.getString("currentGroup", "offline")!!)
        DAO = AbstractProductDAO(dbHelper)

        picturesPath = File(filesDir, "pictures")
        val thumbnailsDir = File(cacheDir, "thumbnails")
        thumbnailsDir.mkdirs()
        picturesPath.mkdirs()
        imageView = findViewById(R.id.imageView)

        saveButton = findViewById(R.id.saveButton)
        takePictureButton = findViewById(R.id.takePictureButton)
        scanButton = findViewById(R.id.scan_button)

        barcodeText = findViewById(R.id.barcodeTextEdit)
        noBarcodeCheckBox = findViewById(R.id.noBarcodeCheckBox)
        productNameText = findViewById(R.id.productName)
        netWeightText = findViewById(R.id.netWeight)
        unitTypeSpinner = findViewById(R.id.unitTypeSpinner)

        categorySpinner = findViewById(R.id.categorySpinner)

        fillupCategorySpinner()
        fillupUnitsSpinner()

        noBarcodeCheckBox.setOnClickListener {
            barcodeText.isEnabled = !noBarcodeCheckBox.isChecked
        }

        barcodeText.addTextChangedListener {
            this.barcode = barcodeText.text.toString()
        }

        val extras = intent.extras
        action = extras!!.get("action") as String
        when (action) {
            "update" -> {
                abstractProduct = extras.get("abstractProduct") as AbstractProduct
            }

            "new_from_barcode" -> {
                abstractProduct = extras.get("abstractProduct") as AbstractProduct
                barcode = abstractProduct!!.barcode
                performRequest(abstractProduct!!.barcode)
            }
        }

        if (abstractProduct != null && action == "update") {
            val imageThumbnailUri = getImageUri(this, File(thumbnailsDir, "${abstractProduct!!.imageHash}.webp"))
            pictureFile = File(picturesPath, "${abstractProduct!!.imageHash}.png")
            imageView.setImageURI(imageThumbnailUri)
            barcodeText.setText(abstractProduct!!.barcode)
            productNameText.text = abstractProduct!!.name
            netWeightText.text = abstractProduct!!.netWeight.toString()
            categorySpinner.setSelection(abstractProduct!!.category - 1)
            unitTypeSpinner.setSelection(abstractProduct!!.unit)
            if (abstractProduct!!.barcode == "" || abstractProduct!!.barcode == "             ") {
                noBarcodeCheckBox.isChecked = true
            }
        }

        saveButton.setOnClickListener {
            val productName = productNameText.text.toString()
            val netWeight = netWeightText.text
            if (action != "update" && (!this::pictureFile.isInitialized || !pictureFile.exists())) {
                Toast.makeText(this, getString(R.string.product_picture_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (barcode == "" && !noBarcodeCheckBox.isChecked) {
                Toast.makeText(this, getString(R.string.product_barcode_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (productName == "") {
                Toast.makeText(this, getString(R.string.product_name_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }
            if (netWeight.toString() == "" || netWeight.toString().toDoubleOrNull() == null) {
                Toast.makeText(this, getString(R.string.product_net_weight_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            loadingDialog.show()

            val currentGroup: Int
            val currentGroupString = sharedPreferences.getString("currentGroup", "offline")!!

            val net = Net()

            if (currentGroupString != "offline") {
                currentGroup = currentGroupString.toInt()

                net.token = sharedPreferences.getString("token", "")!!
                net.server = sharedPreferences.getString("server", "")!!
                net.language = sharedPreferences.getString("language", "en-US")!!

            } else {
                currentGroup = 0
            }

            abstractProduct = AbstractProduct(
                if (abstractProduct == null) 0 else abstractProduct!!.id,
                if (noBarcodeCheckBox.isChecked) "" else barcode,
                productName,
                netWeight.toString().toDouble(),
                pictureFile.nameWithoutExtension,
                categorySpinner.selectedItemPosition + 1,
                unitTypeSpinner.selectedItemPosition
            )

            val pictureFile = File(File(filesDir, "pictures"), "${abstractProduct!!.imageHash}.png")

            if (action == "update") {
                DAO.updateAbstractProduct(abstractProduct!!)
                if (currentGroup > 0) net.updateAbstractProduct(currentGroup, abstractProduct!!, pictureFile, this::notifyUserAndExit)
            } else if (action == "new" || action == "new_from_barcode") {
                abstractProduct!!.id = DAO.addAbstractProduct(abstractProduct!!).toInt()
                if (currentGroup > 0)
                    net.uploadAbstractProduct(currentGroup, abstractProduct!!, pictureFile, this::notifyUserAndExit)
            }
        }

        takePictureButton.setOnClickListener {
            requestPermissionLauncher.launch(android.Manifest.permission.CAMERA)
        }

        scanButton.setOnClickListener {
            scanningBarcode = true
            requestPermissionLauncher.launch(android.Manifest.permission.CAMERA)
        }
    }

    fun notifyUserAndExit(response: Response) {
        runOnUiThread {
            Toast.makeText(this, response.body!!.string(), Toast.LENGTH_LONG).show()
            loadingDialog.dismiss()
            finish()
        }
    }

    fun performRequest(barcode: String) {
        barcodeText.setText(this.barcode)

        var abstractProduct: AbstractProduct

        if (DAO.findAbstractProductByBarcode(this.barcode) != null) {
            AlertDialog.Builder(this)
                .setMessage(getString(R.string.abstract_product_already_exists))
                .setPositiveButton(getString(R.string.quit)) { _: DialogInterface, _: Int ->
                    finish()
                }
                .setNegativeButton(getString(R.string.edit_existing)) { _: DialogInterface, _: Int ->
                    val addProductIntent = Intent(this, AddAbstractProductActivity::class.java)
                    val extras = Bundle()
                    val existingAbstractProduct = DAO.findAbstractProductByBarcode(this.barcode)

                    extras.putParcelable("abstractProduct", existingAbstractProduct)
                    addProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addProductIntent, extras)
                    finish()
                }.show()
        }

        val net = Net();
        net.requestProductFromOnlineDB(barcode, {response ->
            if (response.code == 404) {
                runOnUiThread {
                    Toast.makeText(this, getString(R.string.no_product_in_online_database), Toast.LENGTH_LONG)
                        .show()
                    productNameText.setText("")
                    netWeightText.setText("")
                }
                return@requestProductFromOnlineDB
            }

            abstractProduct = Parser().parse(response.body!!.string())
            runOnUiThread {
                productNameText.text = abstractProduct.name
                netWeightText.text = abstractProduct.netWeight.toString()
                unitTypeSpinner.setSelection(abstractProduct.unit)
            }
        })
    }

    private fun fillupUnitsSpinner() {
        val units = listOf(
            getString(R.string.kilogram),
            getString(R.string.gram),
            getString(R.string.liter),
            getString(R.string.milliliter),
            getString(R.string.pieces)
        )

        val arrayAdapter =
            ArrayAdapter(this, androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, units)
        arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
        unitTypeSpinner.adapter = arrayAdapter
    }

    fun fillupCategorySpinner() {

        val categoriesDAO =
            CategoryDAO(DBStorageController(this, sharedPreferences.getString("currentGroup", "database")!!))

        val categories = categoriesDAO.getAllCategories().map { category -> category.name }

        val arrayAdapter =
            ArrayAdapter(this, androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, categories)
        arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
        categorySpinner.adapter = arrayAdapter
    }

    @RequiresApi(Build.VERSION_CODES.R)
    val takePicture = registerForActivityResult(ActivityResultContracts.TakePicture()) { success: Boolean ->
        if (success) {
            //Move picture to a proper directory according to its calculated hash
            val tempfile = File(filesDir, "image.png")

            val imageContent = tempfile.inputStream().readBytes()
            var img = BitmapFactory.decodeByteArray(imageContent, 0, imageContent.size)

            val exif = ExifInterface(tempfile.absoluteFile.toString())
            val orientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL)
            val matrix = Matrix()

            when (orientation) {
                ExifInterface.ORIENTATION_ROTATE_90 -> matrix.postRotate(90F)
                ExifInterface.ORIENTATION_ROTATE_180 -> matrix.postRotate(180F)
                ExifInterface.ORIENTATION_ROTATE_270 -> matrix.postRotate(270F)
            }
            val rotated = Bitmap.createBitmap(img, 0, 0, img.width, img.height, matrix, true)

            val compressionFactor = sharedPreferences.getInt("compression", 1)
            rotated.compress(
                Bitmap.CompressFormat.WEBP_LOSSY,
                100 * (15 / (16 - compressionFactor)),
                FileOutputStream(tempfile)
            )

            val imageHash = calculateMd5Hash(tempfile)

            pictureFile = File(picturesPath, "$imageHash.png")
            Files.move(tempfile.toPath(), pictureFile.toPath(), StandardCopyOption.REPLACE_EXISTING)
            tempfile.delete()

            generateThumbnailForImage(this, imageHash)

            imageView.setImageURI(getImageUri(this, pictureFile))
        } else {
            Log.e("QWERTYUIOP", "Cannot save a picture")
        }
    }

    @RequiresApi(Build.VERSION_CODES.R)
    fun getPicture() {
        //Saving picture to a temp file for further hash calculation and moving to a proper directory
        val imageFile = File(this.filesDir, "image.png")
        val imageUri = getImageUri(this, imageFile)
        if (imageUri != null) {
            takePicture.launch(imageUri)
        }
    }

    @RequiresApi(Build.VERSION_CODES.R)
    val requestPermissionLauncher =
        registerForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted: Boolean ->
            if (isGranted) {
                if (scanningBarcode) {
                    prepareBarcodeScanner()
                } else {
                    getPicture()
                }
            } else {
                Toast.makeText(this, getString(R.string.camera_permission_for_picture_request), Toast.LENGTH_LONG)
                    .show()
            }
        }

    private val scanLauncher = registerForActivityResult(ScanContract()) { result: ScanIntentResult ->
        run {
            if (result.contents == null) {
                Toast.makeText(this, getString(R.string.cancelled), Toast.LENGTH_SHORT).show()
            } else {
                val scannedBarcode = result.contents
                barcodeText.setText(scannedBarcode)
                performRequest(scannedBarcode)
            }
            scanningBarcode = false
        }
    }

    private fun prepareBarcodeScanner() {
        val options = ScanOptions()
        options.setDesiredBarcodeFormats(ScanOptions.EAN_13)
        options.setPrompt(getString(R.string.scan_barcode_of_a_product))
        options.setCameraId(0)
        options.setBeepEnabled(false)
        options.setBarcodeImageEnabled(true)
        options.setOrientationLocked(false)

        scanLauncher.launch(options)
    }
}