package org.foxarmy.barcodescannerforemployees.activities

import android.app.Activity
import android.app.ProgressDialog
import android.content.SharedPreferences
import android.os.Bundle
import android.widget.Button
import android.widget.EditText
import android.widget.Toast
import okhttp3.Response
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.dataclasses.Category

class AddCategoryActivity : Activity() {

    private lateinit var DAO: CategoryDAO
    private lateinit var sharedPreferences: SharedPreferences
    private lateinit var loadingDialog: ProgressDialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.activity_add_category)

        loadingDialog = ProgressDialog(this)
        loadingDialog.setMessage(getString(R.string.loading_please_wait))
        loadingDialog.setCancelable(false)
        loadingDialog.setTitle(getString(R.string.loading))

        sharedPreferences = org.foxarmy.barcodescannerforemployees.getPreferences(this)

        val dbHelper = DBStorageController(this, sharedPreferences.getString("currentGroup", "offline")!!)
        DAO = CategoryDAO(dbHelper)

        val extras = intent.extras
        val category = extras!!.get("category") as Category?

        val categoryNameTextEdit: EditText = findViewById(R.id.newCategoryName)

        categoryNameTextEdit.setText(category!!.name)

        val net = Net()

        net.token = sharedPreferences.getString("token", "")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.language = sharedPreferences.getString("language", "")!!

        val currentGroup: Int = if (sharedPreferences.getString(
                "currentGroup",
                "offline"
            )!! == "offline"
        ) 0 else sharedPreferences.getString("currentGroup", "")!!.toInt()

        findViewById<Button>(R.id.saveButton).setOnClickListener {
            if (categoryNameTextEdit.text.toString() == "") {
                Toast.makeText(this, getString(R.string.category_name_required), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }
            loadingDialog.show()

            if (category.id == 0) { // Inserting new category
                val newCategory = Category(0, categoryNameTextEdit.text.toString())
                newCategory.id = DAO.addCategory(newCategory).toInt()
                if (currentGroup > 0) net.uploadCategory(currentGroup, newCategory, this::notifyUserAndExit)
            } else { // Updating existing category
                category.name = categoryNameTextEdit.text.toString()
                DAO.updateCategory(category)
                if (currentGroup > 0) net.updateCategory(currentGroup, category, this::notifyUserAndExit)
            }
        }
    }
    fun notifyUserAndExit(response: Response) {
        runOnUiThread {
            Toast.makeText(this, response.body!!.string(), Toast.LENGTH_SHORT).show()
        }
        loadingDialog.dismiss()
        finish()
    }
}