package org.foxarmy.barcodescannerforemployees.activities

import android.app.Activity
import android.app.ProgressDialog
import android.content.Intent
import android.content.SharedPreferences
import android.os.Build
import android.os.Bundle
import android.view.View
import android.widget.*
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.core.widget.addTextChangedListener
import com.journeyapps.barcodescanner.ScanContract
import com.journeyapps.barcodescanner.ScanIntentResult
import com.journeyapps.barcodescanner.ScanOptions
import okhttp3.Response
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.database.ProductDAO
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import org.foxarmy.barcodescannerforemployees.views.AbstractProductView
import java.text.SimpleDateFormat
import java.util.*

class AddProductActivity : AppCompatActivity() {

    private lateinit var scanButton: Button
    private lateinit var noBarcodeButton: Button
    private lateinit var abstractProductView: AbstractProductView
    private lateinit var expiryDateRadioButton: RadioButton
    private lateinit var shelfLifeRadioButton: RadioButton

    private lateinit var expiryDatePicker: DatePicker

    private lateinit var shelfLifeTextEdit: EditText

    private lateinit var amountTextEdit: EditText

    private lateinit var dateOfProductionDatePicker: DatePicker
    private lateinit var saveProductButton: Button

    private var expiryDateOverShelfLife: Boolean? = null
    private var updatingExistentProduct = false
    private var product: Product? = null
    private var abstractProduct: AbstractProduct? = null

    private lateinit var productDAO: ProductDAO
    private lateinit var abstractProductDAO: AbstractProductDAO
    private lateinit var sharedPreferences: SharedPreferences

    private lateinit var loadingDialog: ProgressDialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.fragment_add_product)

        loadingDialog = ProgressDialog(this)
        loadingDialog.setMessage(getString(R.string.loading_please_wait))
        loadingDialog.setCancelable(false)
        loadingDialog.setTitle(getString(R.string.loading))

        sharedPreferences = org.foxarmy.barcodescannerforemployees.getPreferences(this)

        val dbHelper = DBStorageController(this, sharedPreferences.getString("currentGroup", "offline")!!)
        productDAO = ProductDAO(dbHelper)
        abstractProductDAO = AbstractProductDAO(dbHelper)


        scanButton = findViewById(R.id.scanButton)
        noBarcodeButton = findViewById(R.id.noBarcodeButton)
        abstractProductView = findViewById(R.id.abstractProductView)
        expiryDateRadioButton = findViewById(R.id.expiryDateRadio)
        shelfLifeRadioButton = findViewById(R.id.shelfLifeRadio)

        expiryDatePicker = findViewById(R.id.expiryDatePicker)

        shelfLifeTextEdit = findViewById(R.id.shelfLifeTextEdit)

        amountTextEdit = findViewById(R.id.amountTextEdit)
        amountTextEdit.setText("1")

        dateOfProductionDatePicker = findViewById(R.id.dateOfProductionDatePicker)
        saveProductButton = findViewById(R.id.saveProductButton)

        val extras = intent.extras
        product = extras!!.get("product") as Product?

        if (product != null) {
            abstractProduct = abstractProductDAO.findAbstractProductById(product!!.abstractProductId)
            abstractProductView.abstractProduct = abstractProduct!!
            expiryDateRadioButton.isSelected = true
            shelfLifeRadioButton.isSelected = false

            expiryDateOverShelfLife = true
            updatingExistentProduct = true
            update()
        } else {
            product = Product(0, 0, 0, 0, 0)
        }

        scanButton.setOnClickListener {
            requestPermissionLauncher.launch(android.Manifest.permission.CAMERA)
        }

        noBarcodeButton.setOnClickListener {
            intentLauncher.launch(Intent(this, FindBarcodelessAbstractProduct::class.java))
        }

        expiryDateRadioButton.setOnClickListener {
            expiryDateOverShelfLife = true
            update()
        }

        shelfLifeRadioButton.setOnClickListener {
            expiryDateOverShelfLife = false
            update()
        }

        dateOfProductionDatePicker.setOnDateChangedListener { _, year, monthOfYear, dayOfMonth ->
            product!!.dateOfProduction = SimpleDateFormat("dd.MM.yyyy").parse("$dayOfMonth.$monthOfYear.$year")!!.time / 1000
            update()
        }

        expiryDatePicker.setOnDateChangedListener { _, year, monthOfYear, dayOfMonth ->
            product!!.dateOfExpiry = SimpleDateFormat("dd.MM.yyyy").parse("$dayOfMonth.$monthOfYear.$year")!!.time / 1000
            update()
        }

        amountTextEdit.addTextChangedListener {
            val text = amountTextEdit.text.toString()
            if (text == "") return@addTextChangedListener
            product!!.amount = text.toInt()
        }

        val net = Net()

        net.language = sharedPreferences.getString("language", "en-US")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.token = sharedPreferences.getString("token", "")!!

        saveProductButton.setOnClickListener {
            if (abstractProduct == null) {
                Toast.makeText(this, getString(R.string.abstract_product_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (expiryDateOverShelfLife == null) {
                Toast.makeText(this, getString(R.string.shell_life_or_expiry_date_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }
            if (product!!.dateOfProduction == 0.toLong()) {
                Toast.makeText(this, getString(R.string.date_of_production_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (!expiryDateOverShelfLife!!) {
                if (shelfLifeTextEdit.text.toString() != "") {
                    evaluateDateOfExpiry(shelfLifeTextEdit.text.toString().toInt())
                }
            } else {
                if (product!!.dateOfExpiry == 0.toLong()) {
                    Toast.makeText(this, getString(R.string.expiry_date_request), Toast.LENGTH_SHORT).show()
                    return@setOnClickListener
                }
            }

            if (product!!.amount == 0) {
                Toast.makeText(this, getString(R.string.product_amount_request), Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            loadingDialog.show()

            val currentGroup: Int = if (sharedPreferences.getString("currentGroup", "offline")!! == "offline") 0 else sharedPreferences.getString("currentGroup", "")!!.toInt()

            if (updatingExistentProduct) {
                productDAO.updateProduct(product!!)
                if (currentGroup > 0) net.updateProduct(currentGroup, product!!, this::notifyUserAndExit)
            } else {
                product!!.id = productDAO.insertNewProduct(product!!).toInt()
                if (currentGroup > 0) net.uploadProduct(currentGroup, product!!, this::notifyUserAndExit)
            }
        }
        update()

        val today = SimpleDateFormat("dd.MM.yyyy").format(Calendar.getInstance().time).split(".")


        dateOfProductionDatePicker.updateDate(today[2].toInt(), today[1].toInt() - 1, today[0].toInt())
        expiryDatePicker.updateDate(today[2].toInt(), today[1].toInt() - 1, today[0].toInt())
        product!!.dateOfProduction = SimpleDateFormat("dd.MM.yyyy").parse("${today[0]}.${today[1]}.${today[2]}")!!.time / 1000


    }

    fun notifyUserAndExit(response: Response) {
        runOnUiThread {
            Toast.makeText(this, response.body!!.string(), Toast.LENGTH_LONG).show()
            loadingDialog.dismiss()
            finish()
        }
    }

    private val intentLauncher =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == Activity.RESULT_OK) {
                val selectedAbstractProduct = (result.data?.extras!!.getParcelable("abstractProduct") as AbstractProduct?)!!
                product!!.abstractProductId = selectedAbstractProduct.id
                abstractProduct = selectedAbstractProduct
                displayAbstractProduct(selectedAbstractProduct, "")
            }
        }

    private fun update () {
        if (expiryDateOverShelfLife == true) {
            expiryDatePicker.visibility = View.VISIBLE

            shelfLifeTextEdit.visibility = View.INVISIBLE
        } else if (expiryDateOverShelfLife == false){
            expiryDatePicker.visibility = View.INVISIBLE

            shelfLifeTextEdit.visibility = View.VISIBLE
        }

        if (amountTextEdit.text.toString() == "") {
            amountTextEdit.setText(product!!.amount.toString())
        } else {
            product!!.amount = amountTextEdit.text.toString().toInt()
        }

        abstractProductView.update()
    }

    private fun evaluateDateOfExpiry(shelfLifeMonths: Int) {
        if (product!!.dateOfProduction == 0.toLong() ) {
            return
        }

        val c = Calendar.getInstance()
        c.timeInMillis = product!!.dateOfProduction * 1000

        c.add(Calendar.MONTH, shelfLifeMonths)
        product!!.dateOfExpiry = c.timeInMillis / 1000
    }

    private fun displayAbstractProduct(abstractProduct: AbstractProduct?, scannedBarcode: String) {
        if (abstractProduct == null) {
            AlertDialog.Builder(this)
                .setMessage(getString(R.string.abstract_product_does_not_exist))
                .setPositiveButton(R.string.yes) { _, _ ->
                    val addAbstractProductIntent = Intent(this, AddAbstractProductActivity::class.java)
                    val extras = Bundle()
                    extras.putParcelable("abstractProduct", AbstractProduct(0, scannedBarcode, "", 0.0, "", 0, 0))
                    extras.putString("action", "new_from_barcode")
                    addAbstractProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addAbstractProductIntent, extras)
                }
                .setNegativeButton(R.string.no) {_, _ ->

                }.show()
        } else {
            abstractProductView.abstractProduct = abstractProduct
            abstractProductView.update()
        }
    }

    @RequiresApi(Build.VERSION_CODES.R)
    val requestPermissionLauncher =
        registerForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted: Boolean ->
            if (isGranted) {
                prepareBarcodeScanner()
            } else {
                Toast.makeText(this, getString(R.string.camera_permission_for_scanning_request), Toast.LENGTH_LONG).show()
            }
        }

    private val scanLauncher = registerForActivityResult(ScanContract()) { result: ScanIntentResult ->
        run {
            if (result.contents == null) {
                Toast.makeText(this, getString(R.string.cancelled), Toast.LENGTH_SHORT).show()
            } else {
                val scannedBarcode = result.contents
                abstractProduct = abstractProductDAO.findAbstractProductByBarcode(scannedBarcode)

                displayAbstractProduct(abstractProduct, scannedBarcode)
                if (abstractProduct != null) {
                    product?.abstractProductId = abstractProduct!!.id
                }
            }
        }
    }

    private fun prepareBarcodeScanner() {
        val options = ScanOptions()
        options.setDesiredBarcodeFormats(ScanOptions.EAN_13)
        options.setPrompt(getString(R.string.scan_barcode_of_a_product))
        options.setCameraId(0)
        options.setBeepEnabled(false)
        options.setBarcodeImageEnabled(true)
        options.setOrientationLocked(false)

        scanLauncher.launch(options)
    }
}
