package org.foxarmy.barcodescannerforemployees.activities

import android.app.ProgressDialog
import android.content.Intent
import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.databinding.ActivityGroupBinding
import org.foxarmy.barcodescannerforemployees.noInternetConnectionAvailableNotification

class GroupActivity : AppCompatActivity() {

    private lateinit var binding: ActivityGroupBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityGroupBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val loadingDialog = ProgressDialog(this)
        loadingDialog.setMessage(getString(R.string.loading_please_wait))
        loadingDialog.setCancelable(false)
        loadingDialog.setTitle(getString(R.string.loading))

        val sharedPreferences = org.foxarmy.barcodescannerforemployees.getPreferences(this)

        binding.createGroupButton.setOnClickListener {
            val groupName = binding.groupNameTextEdit.text.toString()
            val groupPassword = binding.groupPasswordTextEdit.text.toString()
            val net = Net()
            net.language = sharedPreferences.getString("language", "en-US")!!
            net.server = sharedPreferences.getString("server", "")!!
            net.token = sharedPreferences.getString("token", "")!!

            net.serverIsAvailable(this, { isServerAvailable ->
                if (!isServerAvailable) {
                    noInternetConnectionAvailableNotification(this)
                } else {
                    runOnUiThread {
                        loadingDialog.show()
                    }
                    net.createGroup(groupName, groupPassword, { response ->
                        val responseText = response.body!!.string()

                        if (response.code == 200) {
                            val currentGroups = sharedPreferences.getStringSet("groups", mutableSetOf())
                            currentGroups!!.add(responseText)
                            sharedPreferences.edit().putStringSet("groups", currentGroups).apply()
                            sharedPreferences.edit().putString("currentGroup", responseText).apply()
                            runOnUiThread {
                                val intent = Intent(this, MainActivity::class.java)
                                startActivity(intent)
                                loadingDialog.dismiss()
                                finish()
                            }
                        } else {
                            runOnUiThread {
                                loadingDialog.dismiss()
                                Toast.makeText(this, responseText, Toast.LENGTH_LONG).show()
                            }
                        }
                    })
                }
            })
        }

        binding.joinGroupButton.setOnClickListener {
            val groupName = binding.groupNameTextEdit.text.toString()
            val groupPassword = binding.groupPasswordTextEdit.text.toString()

            val net = Net()
            net.language = sharedPreferences.getString("language", "en-US")!!
            net.server = sharedPreferences.getString("server", "")!!
            net.token = sharedPreferences.getString("token", "")!!

            net.serverIsAvailable(this) { isServerAvailable ->
                if (!isServerAvailable) {
                    noInternetConnectionAvailableNotification(this)
                } else {
                    runOnUiThread {
                        loadingDialog.show()
                    }
                    net.getGroupId(groupName, { requestGroupIdResponse ->
                        var groupId: Int
                        try {
                            groupId = requestGroupIdResponse.toInt()
                        } catch (e: Exception) {
                            runOnUiThread {
                                loadingDialog.dismiss()
                                Toast.makeText(this, requestGroupIdResponse, Toast.LENGTH_SHORT).show()
                            }
                            return@getGroupId
                        }
                        net.joinGroup(groupId, groupPassword, { response ->
                            val responseText = response.body!!.string()

                            if (response.code == 200) {
                                val currentGroups = sharedPreferences.getStringSet("groups", mutableSetOf())
                                currentGroups!!.add(groupId.toString())
                                sharedPreferences.edit().putStringSet("groups", currentGroups).apply()
                                sharedPreferences.edit().putString("currentGroup", groupId.toString()).apply()
                                runOnUiThread {
                                    val intent = Intent(this, MainActivity::class.java)
                                    startActivity(intent)
                                    loadingDialog.dismiss()
                                    finish()
                                }
                            } else {
                                runOnUiThread {
                                    loadingDialog.dismiss()
                                    Toast.makeText(this, responseText, Toast.LENGTH_LONG).show()
                                }
                            }
                        })
                    })
                }
            }
        }
    }
}