package org.foxarmy.barcodescannerforemployees.activities

import android.app.ProgressDialog
import android.content.Intent
import android.os.Bundle
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.databinding.ActivityLoginBinding
import org.foxarmy.barcodescannerforemployees.noInternetConnectionAvailableNotification
import org.foxarmy.barcodescannerforemployees.parseIntArray
import org.json.JSONObject

class LoginActivity : AppCompatActivity() {

    private lateinit var binding: ActivityLoginBinding;

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityLoginBinding.inflate(layoutInflater);
        setContentView(binding.root)

        val loadingDialog = ProgressDialog(this)
        loadingDialog.setMessage(getString(R.string.loading_please_wait))
        loadingDialog.setCancelable(false)
        loadingDialog.setTitle(getString(R.string.loading))

        fillUpLanguagesSpinner()

        val sharedPreferences = org.foxarmy.barcodescannerforemployees.getPreferences(this)

        binding.loginButton.setOnClickListener {
            val server = binding.serverTextEdit.text.toString()
            val username = binding.usernameTextEdit.text.toString()
            val password = binding.passwordTextEdit.text.toString()
            val language = resources.getStringArray(R.array.languages)[binding.languageSpinner.selectedItemPosition]
            sharedPreferences.edit().putString("language", language).apply()

            val net = Net()
            net.language = sharedPreferences.getString("language", "en-US")!!
            net.server = server

            net.serverIsAvailable(this, { isServerAvailable ->
                if (!isServerAvailable) {
                    noInternetConnectionAvailableNotification(this)
                } else {
                    runOnUiThread {
                        loadingDialog.show()
                    }
                    net.login(username, password, { response ->
                        val responseText = response.body!!.string()
                        if (response.code != 200) {
                            runOnUiThread {
                                Toast.makeText(this, responseText, Toast.LENGTH_SHORT).show()
                            }
                        } else {
                            val json = JSONObject(responseText)
                            sharedPreferences.edit().putString("token", json["token"].toString()).apply()
                            net.token = json["token"].toString()
                            sharedPreferences.edit().putInt("userId", json["id"].toString().toInt()).apply()
                            sharedPreferences.edit().putString("server", server).apply()

                            net.getMyGroups({ response ->
                                runOnUiThread {
                                    loadingDialog.dismiss()
                                }
                                val r = response.body!!.string()
                                if (r == "" || r == "[]") {
                                    goToActivity("GroupActivity")
                                }
                                val myGroups = parseIntArray(r).map { a -> a.toString() }

                                sharedPreferences.edit().putStringSet("groups", myGroups.toSet()).apply()
                                sharedPreferences.edit().putString("currentGroup", myGroups[0]).apply()

                                goToActivity("MainActivity")
                            })
                        }
                    })
                }
            })
        }

        binding.registerButton.setOnClickListener {
            val server = binding.serverTextEdit.text.toString()
            val username = binding.usernameTextEdit.text.toString()
            val password = binding.passwordTextEdit.text.toString()
            val language = resources.getStringArray(R.array.languages)[binding.languageSpinner.selectedItemPosition]
            sharedPreferences.edit().putString("language", language).apply()
            sharedPreferences.edit().putString("server", server).apply()

            val net = Net()
            net.language = language
            net.server = server

            net.serverIsAvailable(this, {isServerAvailable ->
                if (!isServerAvailable) {
                    noInternetConnectionAvailableNotification(this)
                } else {
                    net.registerAccount(username, password, {response ->
                        val responseText = response.body!!.string()
                        if (response.code != 200) {
                            Toast.makeText(this, responseText, Toast.LENGTH_SHORT).show();
                        } else {
                            sharedPreferences.edit().putInt("userId", responseText.toInt()).apply()
                            net.login(username, password, {response ->
                                val token = JSONObject(response.body!!.string())["token"].toString()
                                sharedPreferences.edit().putString("token", token).apply()

                                sharedPreferences.edit().putString("server", server).apply()
                                runOnUiThread {
                                    loadingDialog.dismiss()
                                }
                                goToActivity("GroupActivity")
                            })
                        }
                    })
                }
            })
        }

        binding.offlineButton.setOnClickListener {
            sharedPreferences.edit().putString("currentGroup", "offline").apply()
            sharedPreferences.edit().putStringSet("groups", setOf("offline")).apply()

            val intent = Intent(this, MainActivity::class.java)
            startActivity(intent)
            finish()
        }
    }

    private fun fillUpLanguagesSpinner() {
        val languages = resources.getStringArray(R.array.languages)
        val arrayAdapter =
            ArrayAdapter(this, androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, languages)
        arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
        binding.languageSpinner.adapter = arrayAdapter
    }

    private fun goToActivity(activityName: String) {
        runOnUiThread {
            val intent = Intent(this, when(activityName) {
                "MainActivity" -> MainActivity::class.java
                "GroupActivity" -> GroupActivity::class.java
                "LoginActivity" -> LoginActivity::class.java
                else -> LoginActivity::class.java
            })
            startActivity(intent)
            finish()
        }
    }
}