package org.foxarmy.barcodescannerforemployees.activities

import android.app.Activity
import android.app.ProgressDialog
import android.content.DialogInterface
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.ActionBarDrawerToggle
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.drawerlayout.widget.DrawerLayout
import androidx.viewpager.widget.ViewPager
import com.google.android.material.navigation.NavigationView
import org.foxarmy.barcodescannerforemployees.*
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.database.ProductDAO
import org.foxarmy.barcodescannerforemployees.databinding.ActivityMainBinding
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Category
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import org.foxarmy.barcodescannerforemployees.fragments.CategoriesFragment
import org.foxarmy.barcodescannerforemployees.fragments.ShelfFragment
import org.foxarmy.barcodescannerforemployees.fragments.StorageFragment
import org.json.JSONArray
import org.json.JSONObject
import java.io.File


class MainActivity : AppCompatActivity(), NavigationView.OnNavigationItemSelectedListener {

    private lateinit var binding: ActivityMainBinding
    private lateinit var adapter: ViewPagerAdapter
    private var drawerLayout: DrawerLayout? = null
    private var actionBarDrawerToggle: ActionBarDrawerToggle? = null
    private lateinit var ws: WebSocketClient

    var selectionMode = false
    private var selectedAmount = 0

    private lateinit var sharedPreferences: SharedPreferences

    private lateinit var loadingDialog: ProgressDialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        loadingDialog = ProgressDialog(this)
        loadingDialog.setMessage(getString(R.string.loading_please_wait))
        loadingDialog.setCancelable(false)
        loadingDialog.setTitle(getString(R.string.loading))

        sharedPreferences = getPreferences(this)

        setSupportActionBar(binding.toolbar)
        setupViewPager(binding.tabViewpager)
        binding.tabTablayout.setupWithViewPager(binding.tabViewpager)

        binding.tabTablayout.setOnDragListener { _, _ ->
            selectionMode = false
            selectedAmount = 0
            true
        }

        drawerLayout = binding.drawerLayout
        actionBarDrawerToggle = ActionBarDrawerToggle(this, drawerLayout, R.string.nav_open, R.string.nav_close)

        drawerLayout!!.addDrawerListener(actionBarDrawerToggle!!)
        actionBarDrawerToggle!!.syncState()
        binding.navView.setNavigationItemSelectedListener(this)

        supportActionBar!!.setDisplayHomeAsUpEnabled(true)

        binding.expiryCalendarFab.setOnClickListener { _ ->
            val expiryCalendarIntent = Intent(this, ExpiryCalendarActivity::class.java)
            val extras = Bundle()
            expiryCalendarIntent.putExtras(extras)
            ContextCompat.startActivity(this, expiryCalendarIntent, extras)
        }

        binding.newElementFab.setOnClickListener { _ ->
            val currentPosition = binding.tabTablayout.selectedTabPosition
            val fragment = adapter.getItem(currentPosition)

            when (fragment::class.simpleName.toString()) {
                "StorageFragment" -> {
                    val addAbstractProductIntent = Intent(this, AddAbstractProductActivity::class.java)
                    val extras = Bundle()
                    extras.putParcelable("abstractProduct", null)
                    extras.putString("action", "new")
                    addAbstractProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addAbstractProductIntent, extras)
                }

                "CategoriesFragment" -> {
                    val addCategoryIntent = Intent(this, AddCategoryActivity::class.java)
                    val extras = Bundle()
                    extras.putParcelable("category", Category(0, ""))
                    addCategoryIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addCategoryIntent, extras)
                }

                "ShelfFragment" -> {
                    val addProductIntent = Intent(this, AddProductActivity::class.java)
                    val extras = Bundle()
                    extras.putParcelable("product", null)
                    addProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addProductIntent, extras)

                }
            }
        }
        val net = Net()
        net.server = sharedPreferences.getString("server", "")!!
        val currentGroup = sharedPreferences.getString("currentGroup", "offline")!!
        net.serverIsAvailable(this, {isServerAvailable ->
            if ( currentGroup != "offline" && !isServerAvailable) {
                runOnUiThread {
                    noInternetConnectionAvailableNotification(this)
                }
            } else if (currentGroup != "offline" && isServerAvailable) {
                synchronize()
                ws = WebSocketClient(this, sharedPreferences.getString("server", "")!!)
                val token = sharedPreferences.getString("token", "")!!
                ws.connect(token, currentGroup)
            }
        })
    }

    private fun synchronize() {
        if (sharedPreferences.getString("currentGroup", "offline")!! == "offline") return

        val net = Net()

        net.language = sharedPreferences.getString("language", "en-US")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.token = sharedPreferences.getString("token", "")!!

        val currentGroup = sharedPreferences.getString("currentGroup", "")!!.toInt()

        net.synchronize(currentGroup, {response ->
            val data = JSONObject(response.body!!.string())

            val remoteAbstractProducts = data["abstract_products"] as JSONArray
            val remoteProducts = data["products"] as JSONArray
            val remoteCategories = data["categories"] as JSONArray

            val dbHelper = DBStorageController(this, sharedPreferences.getString("currentGroup", "")!!)

            val abstractProductDAO = AbstractProductDAO(dbHelper)
            val productDAO = ProductDAO(dbHelper)
            val categoryDAO = CategoryDAO(dbHelper)

            val localAbstractProducts = abstractProductDAO.getSortedListOfAbstractProducts(0, "", arrayOf(""))
            val localProducts = productDAO.getSortedListOfProducts(0, "", "")
            val localCategories = categoryDAO.getAllCategories()

            syncAbstractProducts(net, currentGroup, abstractProductDAO, remoteAbstractProducts, localAbstractProducts)
            syncProducts(productDAO, remoteProducts, localProducts)
            syncCategories(categoryDAO, remoteCategories, localCategories)
        })
    }

    private fun syncCategories(
        categoryDAO: CategoryDAO,
        remoteCategories: JSONArray,
        localCategories: List<Category>
    ) {
        for (localCategory in localCategories) {
            var found = false
            for (i in 0 until remoteCategories.length()) {
                val remoteProduct = remoteCategories.getJSONObject(i)
                if (remoteProduct["local_id"] == localCategory.id) {
                    found = true
                    break
                }
            }
            if (!found) categoryDAO.eraseCategory(localCategory.id, this)
        }
        for (i in 0 until remoteCategories.length()) {
            var categoryInRemoteDB: JSONObject? = null
            var categoryInLocalDB: Category? = null

            val remoteCategory = remoteCategories.getJSONObject(i)

            for (localCategory in localCategories) {
                if (remoteCategory["local_id"] == localCategory.id) {
                    categoryInRemoteDB = remoteCategory
                    categoryInLocalDB = localCategory
                    break
                }
            }

            if (categoryInLocalDB == null && remoteCategory != null) {
                var newCategory: Category

                with(remoteCategory) {
                    newCategory = Category(
                        this["local_id"].toString().toInt(),
                        this["name"].toString()
                    )
                }

                categoryDAO.addCategory(newCategory)
            }

            if (categoryInRemoteDB != null && categoryInLocalDB != null) {
                if (categoryInRemoteDB["hash"] != categoryInLocalDB.calculateHash()) {
                    val updatedData = Category.createFromJSON(categoryInRemoteDB)

                    categoryDAO.updateCategory(updatedData)
                }
            }
        }
    }

    private fun syncProducts(
        productDAO: ProductDAO,
        remoteProducts: JSONArray,
        localProducts: List<Product>
    ) {
        for (localProduct in localProducts) {
            var found = false
            for (i in 0 until remoteProducts.length()) {
                val remoteProduct = remoteProducts.getJSONObject(i)
                if (remoteProduct["local_id"] == localProduct.id) {
                    found = true
                    break
                }
            }
            if (!found) productDAO.eraseProduct(localProduct.id)
        }
        for (i in 0 until remoteProducts.length()) {
            var productInRemoteDB: JSONObject? = null
            var productInLocalDB: Product? = null

            val remoteProduct = remoteProducts.getJSONObject(i)

            for (localProduct in localProducts) {
                if (remoteProduct["local_id"] == localProduct.id) {
                    productInRemoteDB = remoteProduct
                    productInLocalDB = localProduct
                    break
                }
            }

            if (productInLocalDB == null && remoteProduct != null) {
                val newProduct = Product.createFromJSON(remoteProduct)

                productDAO.insertNewProduct(newProduct)
            }

            if (productInRemoteDB != null && productInLocalDB != null) {
                if (productInRemoteDB["hash"] != productInLocalDB.calculateHash()) {
                    val updatedData = Product.createFromJSON(productInRemoteDB)

                    productDAO.updateProduct(updatedData)
                }
            }
        }
    }

    private fun syncAbstractProducts(
        net: Net,
        currentGroup: Int,
        abstractProductDAO: AbstractProductDAO,
        remoteAbstractProducts: JSONArray,
        localAbstractProducts: List<AbstractProduct>
    ) {
        for (localAbstractProduct in localAbstractProducts) {
            var found = false
            for (i in 0 until remoteAbstractProducts.length()) {
                val remoteAbstractProduct = remoteAbstractProducts.getJSONObject(i)
                if (remoteAbstractProduct["local_id"] == localAbstractProduct.id) {
                    found = true
                    break
                }
            }
            if (!found) abstractProductDAO.eraseAbstractProduct(localAbstractProduct.id, this)
        }
        for (i in 0 until remoteAbstractProducts.length()) {
            var abstractProductInRemoteDB: JSONObject? = null
            var abstractProductInLocalDB: AbstractProduct? = null

            val remoteAbstractProduct = remoteAbstractProducts.getJSONObject(i)

            for (localAbstractProduct in localAbstractProducts) {
                if (remoteAbstractProduct["local_id"] == localAbstractProduct.id) {
                    abstractProductInRemoteDB = remoteAbstractProduct
                    abstractProductInLocalDB = localAbstractProduct
                    break
                }
            }

            if (abstractProductInLocalDB == null && remoteAbstractProduct != null) {
                val localId = remoteAbstractProduct["local_id"].toString().toInt()

                val picturesDir = File(filesDir, "pictures")
                picturesDir.mkdirs()

                val pictureFile =
                    File(picturesDir, "${remoteAbstractProduct["image_filename"]}.png")
                val url = "https://${net.server}/api/abstractproduct/getImage/${currentGroup}/${localId}"
                net.downloadImage(url, pictureFile, {
                    val newAbstractProduct = AbstractProduct.createFromJSON(remoteAbstractProduct)

                    abstractProductDAO.addAbstractProduct(newAbstractProduct)
                })
            }

            if (abstractProductInRemoteDB != null && abstractProductInLocalDB != null) {
                if (abstractProductInRemoteDB["hash"] != abstractProductInLocalDB.calculateHash()) {
                    val localId = abstractProductInRemoteDB["local_id"].toString().toInt()

                    val pictureFile =
                        File(File(filesDir, "pictures"), "${abstractProductInRemoteDB["image_filename"]}.png")
                    val url = "https://${net.server}/api/abstractproduct/getImage/${currentGroup}/${localId}"
                    net.downloadImage(url, pictureFile, {
                        val updatedData = AbstractProduct.createFromJSON(abstractProductInRemoteDB)

                        abstractProductDAO.updateAbstractProduct(updatedData)
                    })
                }
            }
        }
    }

    private fun setupViewPager(viewpager: ViewPager) {
        adapter = ViewPagerAdapter(supportFragmentManager)

        adapter.addFragment(StorageFragment.newInstance("", arrayOf("")), getString(R.string.storage_title))
        adapter.addFragment(ShelfFragment(), getString(R.string.shelf_title))
        adapter.addFragment(CategoriesFragment(), getString(R.string.categories_title))

        viewpager.adapter = adapter
        viewpager.offscreenPageLimit = 3
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)

        return true
    }

    private fun isOffline(): Boolean {
        return sharedPreferences.getString("currentGroup", "") == "offline"
    }

    override fun onNavigationItemSelected(item: MenuItem): Boolean {
        lateinit var intent: Intent
        when (item.itemId) {
            R.id.nav_account -> {
                if (isOffline()) {
                    Toast.makeText(this, getString(R.string.online_only_feature), Toast.LENGTH_SHORT).show()
                    return false
                }
                intent = Intent(this, AccountSettingsActivity::class.java)
                settingsActivityLauncher.launch(intent)
                return true
            }

            R.id.nav_groups -> {
                if (isOffline()) {
                    Toast.makeText(this, getString(R.string.online_only_feature), Toast.LENGTH_SHORT).show()
                    return false
                }
                intent = Intent(this, MyGroupsActivity::class.java)
            }

            R.id.nav_settings -> {
                intent = Intent(this, SettingsActivity::class.java)
            }

            R.id.nav_logout -> {
                sharedPreferences.edit().putString("currentGroup", "").apply()
                sharedPreferences.edit().putStringSet("groups", emptySet()).apply()
                sharedPreferences.edit().putString("token", "").apply()
                sharedPreferences.edit().putString("server", "").apply()

                intent = Intent(this, LoginActivity::class.java)
            }
        }
        startActivity(intent)

        return true
    }

    private val settingsActivityLauncher = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
        if (result.resultCode == Activity.RESULT_OK) {
            synchronize()
            updateAll()
        }/* else if (result.resultCode == Activity.RESULT_CANCELED) {

        }*/
    }

    fun updateAll() {
        runOnUiThread {
            try {
                val storageFragment: StorageFragment = adapter.getItem(0) as StorageFragment
                val shelfFragment: ShelfFragment = adapter.getItem(1) as ShelfFragment
                val categoriesFragment: CategoriesFragment = adapter.getItem(2) as CategoriesFragment

                storageFragment.updateContent()
                shelfFragment.updateContent()
                categoriesFragment.updateContent()
            } catch (e:Exception) {
                Log.e("BSFE/MainActivity", e.message!!)
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (actionBarDrawerToggle!!.onOptionsItemSelected(item)) {
            return true
        } else {

            val currentPosition = binding.tabTablayout.selectedTabPosition
            val fragment = adapter.getItem(currentPosition)

            return when (item.itemId) {
                R.id.action_settings -> {
                    val intent = Intent(this, SettingsActivity::class.java)
                    settingsActivityLauncher.launch(intent)
                    true
                }

                R.id.action_delete -> {
                    when (fragment::class.simpleName.toString()) {
                        "StorageFragment" -> {
                            AlertDialog.Builder(this)
                                .setMessage(getString(R.string.deleting_abstract_product_warning))
                                .setPositiveButton(getString(R.string.yes)) { _: DialogInterface, _: Int ->
                                    val storageFragment = fragment as StorageFragment
                                    storageFragment.removeSelected()
                                }
                                .setNegativeButton(getString(R.string.no)) { _: DialogInterface, _: Int ->

                                }.show()
                        }

                        "CategoriesFragment" -> {
                            AlertDialog.Builder(this)
                                .setMessage(getString(R.string.deleting_category_warning))
                                .setPositiveButton(getString(R.string.yes)) { _: DialogInterface, _: Int ->
                                    val categoriesFragment = fragment as CategoriesFragment
                                    categoriesFragment.removeSelected()
                                }
                                .setNegativeButton(getString(R.string.no)) { _: DialogInterface, _: Int ->

                                }.show()
                        }

                        "ShelfFragment" -> {
                            val shelfFragment = fragment as ShelfFragment
                            shelfFragment.removeSelected()
                        }
                    }
                    true
                }

                R.id.action_update -> {
                    when (fragment::class.simpleName.toString()) {
                        "StorageFragment" -> {
                            val storageFragment = fragment as StorageFragment
                            storageFragment.updateSelected()
                        }

                        "CategoriesFragment" -> {
                            val categoriesFragment = fragment as CategoriesFragment
                            categoriesFragment.updateSelected()
                        }

                        "ShelfFragment" -> {
                            val shelfFragment = fragment as ShelfFragment
                            shelfFragment.updateSelected()
                        }
                    }
                    true
                }

                else -> super.onOptionsItemSelected(item)
            }
        }

    }

    fun filterAbstractProductsByCategory(id: Int) {
        binding.tabViewpager.setCurrentItem(0, true)

        val currentPosition = binding.tabTablayout.selectedTabPosition
        val fragment = adapter.getItem(currentPosition)

        val storageFragment = fragment as StorageFragment
        storageFragment.filterByCategory(id)
    }

    fun addSelection() {
        selectionMode = true
        selectedAmount ++
    }

    fun removeSelection() {
        selectedAmount --
        if (selectedAmount <= 0) {
            selectedAmount = 0
            selectionMode = false
        }
    }
}