package org.foxarmy.barcodescannerforemployees.activities

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.view.View
import android.widget.EditText
import android.widget.PopupMenu
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.databinding.ActivityManageGroupBinding
import org.foxarmy.barcodescannerforemployees.parseIntArray
import org.foxarmy.barcodescannerforemployees.views.GroupMemberView

class ManageGroupActivity : AppCompatActivity() {
    private lateinit var binding: ActivityManageGroupBinding
    private var isAdmin: Boolean = false
    private lateinit var sharedPreferences: SharedPreferences
    private lateinit var net: Net
    private var groupId: Int = 0

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityManageGroupBinding.inflate(layoutInflater)
        setContentView(binding.root)

        groupId = intent.extras!!.getInt("groupId")

        sharedPreferences = org.foxarmy.barcodescannerforemployees.getPreferences(this)

        net = Net()

        net.token = sharedPreferences.getString("token", "")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.language = sharedPreferences.getString("language", "en-US")!!


        setUpMembers()

        binding.renameButton.setOnClickListener {
            val input = EditText(this)
            AlertDialog.Builder(this)
                .setTitle(getString(R.string.new_group_name))
                .setView(input)
                .setPositiveButton(getString(R.string.ok)) { _, _ ->
                    val newName = input.text.toString()
                    net.renameGroup(groupId, newName, { response ->
                        runOnUiThread {
                            Toast.makeText(this, response.body!!.string(), Toast.LENGTH_LONG).show()
                        }
                    })

                }
                .setNegativeButton(getString(R.string.cancel)) { _, _ ->

                }.show()
        }

        binding.leaveButton.setOnClickListener {
            net.leaveGroup(groupId, { response ->
                runOnUiThread {
                    Toast.makeText(this, response.body!!.string(), Toast.LENGTH_SHORT).show()
                }
                if (response.code == 200) {
                    val groups = sharedPreferences.getStringSet("groups", emptySet())!!
                    groups.remove(groupId.toString())
                    sharedPreferences.edit().putStringSet("groups", groups).apply()
                    sharedPreferences.edit().putString("currentGroup", "").apply()

                    if (groups.isEmpty()) {
                        runOnUiThread {
                            val intent = Intent(this, GroupActivity::class.java)
                            startActivity(intent)
                            finish()
                        }
                    }
                    sharedPreferences.edit().putString("currentGroup", groups.toList()[0]).apply()
                    runOnUiThread {
                        finish()
                    }
                }
            })
        }

        setUpAdminRelatedButtons()
    }

    private fun setUpMembers() {
        net.getUsersInGroup(groupId, { response ->
            val users = parseIntArray(response.body!!.string())
            for (user in users) {
                net.getUsernameById(user, { response ->
                    val groupMemberView = GroupMemberView(this, this as Context, response.body!!.string(), user)

                    groupMemberView.setOnLongClickListener { view ->
                        amIAnAdminIn(groupId, { amIAnAdmin ->
                            if (!amIAnAdmin) return@amIAnAdminIn
                            runOnUiThread {
                                val popupMenu = PopupMenu(this, groupMemberView)
                                popupMenu.inflate(R.menu.user_pop_menu)
                                popupMenu.setOnMenuItemClickListener { item ->
                                    when (item.itemId) {
                                        R.id.kick -> {

                                            setUpMembers()
                                            false
                                        }

                                        R.id.transfer_ownership -> {
                                            AlertDialog.Builder(this)
                                                .setMessage(getString(R.string.transfer_ownership_confirmation))
                                                .setPositiveButton(R.string.yes) { _, _ ->
                                                    net.transfer_ownership(
                                                        groupId,
                                                        (view as GroupMemberView).userId,
                                                        { response ->
                                                            runOnUiThread {
                                                                Toast.makeText(
                                                                    this,
                                                                    response.body!!.string(),
                                                                    Toast.LENGTH_SHORT
                                                                ).show()
                                                                setUpAdminRelatedButtons()
                                                            }
                                                        })

                                                }
                                                .setNegativeButton(R.string.no) { _, _ -> }
                                                .show()
                                            true
                                        }

                                        else -> false
                                    }
                                }
                                runOnUiThread {
                                    popupMenu.show()
                                }
                            }
                        })
                        true
                    }
                    runOnUiThread {
                        binding.groupsContent.addView(groupMemberView)
                    }
                })
            }
        })
    }

    fun amIAnAdminIn(groupId: Int, callback: (Boolean) -> Unit) {
        val net = Net()

        net.token = sharedPreferences.getString("token", "")!!
        net.server = sharedPreferences.getString("server", "")!!
        net.language = sharedPreferences.getString("language", "en-US")!!

        net.getGroupAdminId(groupId, { response ->
            val result = sharedPreferences.getInt("userId", 0) == response.body!!.string().toInt()
            callback(result)
        })
    }

    fun setUpAdminRelatedButtons() {
        amIAnAdminIn(groupId, { isAdmin ->
            runOnUiThread {
                if (!isAdmin) {
                    binding.renameButton.visibility = View.GONE
                }
            }
        })
    }
}