package org.foxarmy.barcodescannerforemployees.activities

import android.app.Activity
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.parseStringList

class NavigatorActivity : Activity() {

    private lateinit var sharedPreferences: SharedPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        sharedPreferences = org.foxarmy.barcodescannerforemployees.getPreferences(this)

        var intent = Intent(this, MainActivity::class.java)

        isInGroup { flag ->
            if (!flag && !isOffline()) {
                intent = Intent(this, GroupActivity::class.java)
            }
            if (!isAuthenticated() && !isOffline()) {
                intent = Intent(this, LoginActivity::class.java);
            }

            runOnUiThread {
                startActivity(intent);
                finish();
            }
        }
    }

    private fun isOffline(): Boolean {
        return sharedPreferences.getString("currentGroup", "")!! == "offline"
    }

    private fun isAuthenticated(): Boolean {
        return sharedPreferences.getString("token", "") != ""
    }

    private fun isInGroup(callback: (Boolean) -> Unit) {
        val groups = sharedPreferences.getStringSet("groups", emptySet())!!
        if (groups.isEmpty()) {
            if (sharedPreferences.getString("token", "")!! == "") callback(false)
            val net = Net()
            net.language = sharedPreferences.getString("language", "")!!
            net.server = sharedPreferences.getString("server", "")!!
            net.token = sharedPreferences.getString("token", "")!!

            net.getMyGroups { response ->
                val responseBody = response.body!!.string()
                if (responseBody == "" || responseBody == "[]") callback(false)
                val groupsFromServer = parseStringList(responseBody)
                if (groupsFromServer.isNotEmpty()) {
                    sharedPreferences.edit().putStringSet("groups", groupsFromServer.toSet()).apply()
                    sharedPreferences.edit().putString("currentGroup", groupsFromServer[0]).apply()
                    callback(true)
                } else {
                    callback(false)
                }
            }
        }
        callback(true)
    }
}