package org.foxarmy.barcodescannerforemployees.activities

import android.app.Activity
import android.content.SharedPreferences
import android.os.Bundle
import android.view.View
import android.widget.ArrayAdapter
import androidx.appcompat.app.AppCompatActivity
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.databinding.ActivitySettingsBinding

class SettingsActivity : AppCompatActivity() {
    lateinit var binding: ActivitySettingsBinding
    lateinit var sharedPreferences: SharedPreferences
    lateinit var myGroups: List<String>
    lateinit var currentGroup: String
    lateinit var groupsNames: MutableList<String>
    lateinit var namesMap: MutableMap<Int, String>

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        sharedPreferences = org.foxarmy.barcodescannerforemployees.getPreferences(this)

        myGroups = sharedPreferences.getStringSet("groups", emptySet())!!.toList()
        currentGroup = sharedPreferences.getString("currentGroup", "offline")!!
        val net = Net()

        net.token = sharedPreferences.getString("token", "")!!
        net.language = sharedPreferences.getString("language", "")!!
        net.server = sharedPreferences.getString("server", "")!!

        binding = ActivitySettingsBinding.inflate(layoutInflater)
        setContentView(binding.root)

        setUpImageCompressionSeekBar()


        if (isOffline()) {
            binding.currentGroupSetting.visibility = View.GONE
        } else {
            namesMap = mutableMapOf()

            for (myGroup in myGroups) {
                net.getGroupName(myGroup.toInt(), { name ->
                    namesMap[myGroup.toInt()] = name
                    fillUpCurrentGroupSpinner()
                })
            }
        }

        binding.saveButton.setOnClickListener {
            net.getGroupId(binding.currentGroupSpinner.selectedItem.toString(), { groupId ->
                sharedPreferences.edit().putString("currentGroup", groupId).apply()
                sharedPreferences.edit().putInt("imageCompression", binding.imageCompressionFactorSeekBar.progress + 1)
                    .apply()
                runOnUiThread {
                    setResult(Activity.RESULT_OK)
                    finish()
                }
            })
        }

        binding.cancelButton.setOnClickListener {
            setResult(Activity.RESULT_CANCELED)
            finish()
        }
    }

    private fun isOffline(): Boolean {
        return sharedPreferences.getString("currentGroup", "offline") == "offline"
    }

    private fun setUpImageCompressionSeekBar() {
        val compressionFactor = sharedPreferences.getInt("imageCompression", 1)
        binding.imageCompressionFactorSeekBar.progress = compressionFactor - 1
    }

    private fun fillUpCurrentGroupSpinner() {
        runOnUiThread {
            if (currentGroup == "offline") {
                binding.currentGroupSetting.visibility = View.GONE
            }
            groupsNames = mutableListOf()
            groupsNames.clear()
            val sortedMap = namesMap.entries.sortedBy { it.key }.associate { it.toPair() }
            for ((id, name) in sortedMap) {
                groupsNames.add(name)
            }

            val arrayAdapter =
                ArrayAdapter(this, androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, groupsNames)
            arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
            binding.currentGroupSpinner.adapter = arrayAdapter
            if (myGroups.indexOf(currentGroup) < groupsNames.size) binding.currentGroupSpinner.setSelection(
                myGroups.indexOf(
                    currentGroup
                )
            )
        }

    }
}