package org.foxarmy.barcodescannerforemployees.database

import android.content.ContentValues
import android.content.Context
import android.database.DatabaseUtils
import android.provider.BaseColumns
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import java.io.File

class AbstractProductDAO(private val dbHelper: DBStorageController) {

    fun getAllAbstractProductInCategory(id: Int): List<AbstractProduct> {

        val db = dbHelper.readableDatabase

        val result = mutableListOf<AbstractProduct>()

        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.BARCODE,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
            AbstractProductContract.AbstractProductEntry.UNIT
        )

        val selection = "${AbstractProductContract.AbstractProductEntry.CATEGORY} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )

        with(cursor) {
            while (moveToNext()) {
                val abstractProductId = getInt(getColumnIndexOrThrow(android.provider.BaseColumns._ID))
                val abstractProductBarcode =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.BARCODE))
                val abstractProductName =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val abstractProductNetWeight =
                    getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val abstractProductImageHash =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                val abstractProductUnit =
                    getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.UNIT))

                val abstractProduct = AbstractProduct(
                    abstractProductId,
                    abstractProductBarcode,
                    abstractProductName,
                    abstractProductNetWeight,
                    abstractProductImageHash,
                    category = id,
                    abstractProductUnit
                )

                result.add(abstractProduct)
            }
        }

        return result
    }

    fun getAmountOfAbstractProductsInCategory(id: Int): Int {
        val db = dbHelper.readableDatabase
        return DatabaseUtils.longForQuery(
            db,
            "SELECT COUNT(*) FROM ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} WHERE ${AbstractProductContract.AbstractProductEntry.CATEGORY} = ?",
            arrayOf(id.toString())
        ).toInt()
    }

    fun eraseAbstractProduct(id: Int, context: Context) {

        val db = dbHelper.writableDatabase

        val projectionForAbstractProduct = arrayOf(
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME
        )

        val selectionForAbstractProduct = "${BaseColumns._ID} = ?"
        val selectionArgsForAbstractProduct = arrayOf(id.toString())
        val cursorForAbstractProduct = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projectionForAbstractProduct,
            selectionForAbstractProduct,
            selectionArgsForAbstractProduct,
            null,
            null,
            null
        )
        var imageHash: String = ""

        with(cursorForAbstractProduct) {
            while (moveToNext()) {
                val productImageHash =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                imageHash = productImageHash
            }
        }
        val picturesDir = File(context.filesDir, "pictures")
        val thumbnailsDir = File(context.cacheDir, "thumbnails")

        File(picturesDir, "$imageHash.png").delete()
        File(thumbnailsDir, "$imageHash.webp").delete()

        val projectionForProducts = arrayOf(BaseColumns._ID)
        val selectionForProducts = "${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID} = ?"
        val selectionArgsForProducts = arrayOf(id.toString())

        val cursorForProducts = db.query(
            ProductContract.ProductEntry.TABLE_NAME,
            projectionForProducts,
            selectionForProducts,
            selectionArgsForProducts,
            null,
            null,
            null
        )

        with(cursorForProducts) {
            while (moveToNext()) {
                ProductDAO(dbHelper).eraseProduct(getInt(getColumnIndexOrThrow(android.provider.BaseColumns._ID)))
            }
        }

        db.delete(AbstractProductContract.AbstractProductEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    fun findAbstractProductByBarcode(barcode: String): AbstractProduct? {

        val db = dbHelper.readableDatabase

        var abstractProduct: AbstractProduct? = null
        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
            AbstractProductContract.AbstractProductEntry.CATEGORY,
            AbstractProductContract.AbstractProductEntry.UNIT
        )

        val selection = "${AbstractProductContract.AbstractProductEntry.BARCODE} = ?"
        val selectionArgs = arrayOf(barcode)

        val cursor = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )

        with(cursor) {
            while (moveToNext()) {
                val id = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val productName =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val imageFilename =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                val netWeight =
                    getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val category = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.CATEGORY))
                val unit = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.UNIT))

                abstractProduct = AbstractProduct(id, barcode, productName, netWeight, imageFilename, category, unit)
            }
        }

        return abstractProduct
    }

    fun findAbstractProductById(id: Int): AbstractProduct? {

        val db = dbHelper.readableDatabase

        var abstractProduct: AbstractProduct? = null
        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.CATEGORY,
            AbstractProductContract.AbstractProductEntry.BARCODE,
            AbstractProductContract.AbstractProductEntry.UNIT
        )

        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )

        with(cursor) {
            while (moveToNext()) {
                val productId = getInt(getColumnIndexOrThrow(android.provider.BaseColumns._ID))
                val barcode = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.BARCODE))
                val name = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val netWeight =
                    getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val imageHash =
                    getString((getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME)))
                val category = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.CATEGORY))
                val unit = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.UNIT))

                abstractProduct = AbstractProduct(productId, barcode, name, netWeight, imageHash, category, unit)
            }
        }

        return abstractProduct
    }

    fun addAbstractProduct(abstractProduct: AbstractProduct): Long {

        val db = dbHelper.writableDatabase

        val values = ContentValues().apply {
            if (abstractProduct.id > 0) put(BaseColumns._ID, abstractProduct.id)
            put(AbstractProductContract.AbstractProductEntry.BARCODE, abstractProduct.barcode)
            put(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME, abstractProduct.name)
            put(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT, abstractProduct.netWeight.toString())
            put(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME, abstractProduct.imageHash)
            put(AbstractProductContract.AbstractProductEntry.CATEGORY, abstractProduct.category)
            put(AbstractProductContract.AbstractProductEntry.UNIT, abstractProduct.unit)
        }

        val id = db.insert(AbstractProductContract.AbstractProductEntry.TABLE_NAME, null, values)
        return id
    }

    fun updateAbstractProduct(abstractProduct: AbstractProduct) {

        val db = dbHelper.writableDatabase

        val values = ContentValues().apply {
            put(BaseColumns._ID, abstractProduct.id)
            put(AbstractProductContract.AbstractProductEntry.BARCODE, abstractProduct.barcode)
            put(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME, abstractProduct.name)
            put(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT, abstractProduct.netWeight.toString())
            put(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME, abstractProduct.imageHash)
            put(AbstractProductContract.AbstractProductEntry.CATEGORY, abstractProduct.category)
            put(AbstractProductContract.AbstractProductEntry.UNIT, abstractProduct.unit)
        }

        db.update(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            values,
            "${BaseColumns._ID} = ?",
            arrayOf(abstractProduct.id.toString())
        )
    }

    fun getSortedListOfAbstractProducts(
        selectedSort: Int,
        filterBy: String,
        filter: Array<String>
    ): List<AbstractProduct> {

        val db = dbHelper.readableDatabase

        val abstractProducts = mutableListOf<AbstractProduct>()

        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.BARCODE,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.CATEGORY,
            AbstractProductContract.AbstractProductEntry.UNIT
        )

        var orderBy: String = ""
        when (selectedSort) {
            0 -> {
                orderBy = "${AbstractProductContract.AbstractProductEntry.PRODUCT_NAME} ASC"
            }

            1 -> {
                orderBy = "${AbstractProductContract.AbstractProductEntry.CATEGORY} ASC"
            }
        }

        var selection = ""
        var selectionArgs: Array<String>? = null

        when (filterBy) {
            "category" -> {
                selection = "${AbstractProductContract.AbstractProductEntry.CATEGORY} = ?"
                selectionArgs = filter
            }

            "barcodeless" -> {
                selection = "${AbstractProductContract.AbstractProductEntry.BARCODE} = ? OR ${AbstractProductContract.AbstractProductEntry.BARCODE} = ? OR ${AbstractProductContract.AbstractProductEntry.BARCODE} = ?"
                selectionArgs = filter
            }
        }


        val cursor = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            orderBy
        )

        with(cursor) {
            while (moveToNext()) {
                val productId = getInt(getColumnIndexOrThrow(android.provider.BaseColumns._ID))
                val barcode = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.BARCODE))
                val productName =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val netWeight =
                    getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val productImageHash =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                val category = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.CATEGORY))
                val unit = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.UNIT))

                val product =
                    AbstractProduct(productId, barcode, productName, netWeight, productImageHash, category, unit)

                abstractProducts.add(product)
            }
        }
        return abstractProducts
    }
}