package org.foxarmy.barcodescannerforemployees.fragments

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.core.view.children
import androidx.fragment.app.Fragment
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.activities.AddCategoryActivity
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.getPreferences
import org.foxarmy.barcodescannerforemployees.views.CategoryView
import kotlin.concurrent.thread

class CategoriesFragment : Fragment() {

    private lateinit var sharedPreferences: SharedPreferences
    private lateinit var categoryDAO: CategoryDAO
    private var updateInProgress = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        sharedPreferences = getPreferences(requireContext())

        prepareDatabaseConnection()
    }

    private fun prepareDatabaseConnection() {
        val dbHelper = DBStorageController(requireContext(), sharedPreferences.getString("currentGroup", "offline")!!)
        categoryDAO = CategoryDAO(dbHelper)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_categories, container, false)
    }

    override fun onResume() {
        super.onResume()

        updateContent()
    }

    fun removeSelected() {
        thread {
            val layout = view?.findViewById<LinearLayout>(R.id.categoriesLayout)

            var deleted = false
            for (view: CategoryView in layout?.children!!.iterator() as Iterator<CategoryView>) {
                if (view.isCategorySelected) {
                    val net = Net()
                    net.token = sharedPreferences.getString("token", "")!!
                    net.server = sharedPreferences.getString("server", "")!!
                    net.language = sharedPreferences.getString("language", "en-US")!!
                    val currentGroup = sharedPreferences.getString("currentGroup", "")!!.toInt()

                    categoryDAO.eraseCategory(view.category.id, requireContext())
                    net.deleteCategory(currentGroup, view.category.id, {response ->
                        activity!!.runOnUiThread{
                            Toast.makeText(context, response.body!!.string(), Toast.LENGTH_SHORT).show()
                        }

                    })
                    deleted = true
                }
            }

            if (!deleted) {
                activity!!.runOnUiThread {
                    Toast.makeText(requireContext(), getString(R.string.nothing_to_delete), Toast.LENGTH_SHORT).show()
                }
            }
            updateContent()
        }
    }

    fun updateSelected() {
        thread {
            val layout = view?.findViewById<LinearLayout>(R.id.categoriesLayout)
            for (view: CategoryView in layout?.children!!.iterator() as Iterator<CategoryView>) {
                if (view.isCategorySelected) {
                    val addCategoryIntent = Intent(context, AddCategoryActivity::class.java)
                    val extras = Bundle()
                    extras.putParcelable("category", view.category)
                    addCategoryIntent.putExtras(extras)
                    activity!!.runOnUiThread {
                        ContextCompat.startActivity(context!!, addCategoryIntent, extras)
                    }
                }
            }
        }
    }

    fun updateContent() {
        thread {
            if (updateInProgress) return@thread
            updateInProgress = true
            prepareDatabaseConnection()
            val layout = view?.findViewById<LinearLayout>(R.id.categoriesLayout)
            activity!!.runOnUiThread {
                layout?.removeAllViews()
            }

            val categories = categoryDAO.getAllCategories()

            for (category in categories) {
                val categoryView = CategoryView(requireActivity(), requireContext(), category)
                activity!!.runOnUiThread {
                    layout?.addView(categoryView)
                }
            }

            updateInProgress = false
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        updateContent()
    }
}