package org.foxarmy.barcodescannerforemployees.fragments

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.ImageView
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.core.view.children
import androidx.fragment.app.Fragment
import androidx.gridlayout.widget.GridLayout
import org.foxarmy.barcodescannerforemployees.Net
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.activities.AddProductActivity
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.database.ProductDAO
import org.foxarmy.barcodescannerforemployees.databinding.FragmentShelfBinding
import org.foxarmy.barcodescannerforemployees.getPreferences
import org.foxarmy.barcodescannerforemployees.views.ProductView
import kotlin.concurrent.thread

class ShelfFragment : Fragment() {

    private lateinit var binding: FragmentShelfBinding
    private var updateInProgress = false
    private var filterBy = ""
    private var filter = ""

    private lateinit var productDAO: ProductDAO
    private lateinit var sharedPreferences: SharedPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        sharedPreferences = getPreferences(requireContext())

        prepareDatabaseConnection()
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentShelfBinding.inflate(layoutInflater)

        fillUpSortBySpinner()

        val filterByDate = arguments?.getLong("filterByExpiryDate") as Long?
        if (filterByDate != null) {
            filter = filterByDate.toString()
            filterBy = "expiryDate"
        }

        binding.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(parent: AdapterView<*>?) {
                updateContent()
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                updateContent()
            }
        }

        updateContent()

        return binding.root
    }

    private fun prepareDatabaseConnection() {
        val dbHelper = DBStorageController(requireContext(), sharedPreferences.getString("currentGroup", "database")!!)
        productDAO = ProductDAO(dbHelper)
    }

    fun updateSelected() {
        val grv = view?.findViewById<GridLayout>(R.id.contentGridLayout)

        var updated = false
        for (view: ProductView in grv?.children!!.iterator() as Iterator<ProductView>) {
            if (view.isProductSelected) {
                val addProductIntent = Intent(requireContext(), AddProductActivity::class.java)
                val extras = Bundle()
                extras.putParcelable("product", view.product)
                addProductIntent.putExtras(extras)
                ContextCompat.startActivity(requireContext(), addProductIntent, extras)

                updated = true
            }
        }

        if (!updated) {
            Toast.makeText(requireContext(), getString(R.string.nothing_to_update), Toast.LENGTH_SHORT).show()
        }
        updateContent()
    }

    private fun fillUpSortBySpinner() {
        val sorts = resources.getStringArray(R.array.product_sort_types)

        val arrayAdapter =
            ArrayAdapter(requireContext(), androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, sorts)
        arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
        binding.spinner.adapter = arrayAdapter
    }

    override fun onResume() {
        super.onResume()

        updateContent()
    }

    fun updateContent() {
        thread {
            if (updateInProgress) return@thread
            updateInProgress = true
            prepareDatabaseConnection()
            val grv = view?.findViewById<GridLayout>(R.id.contentGridLayout)

            activity!!.runOnUiThread {
                grv?.removeAllViews()
            }

            val products = productDAO.getSortedListOfProducts(binding.spinner.selectedItemPosition, filterBy, filter)

            for (product in products.iterator()) {
                val productView = ProductView(
                    requireActivity(),
                    requireContext(),
                    product
                )
                activity!!.runOnUiThread {
                    grv?.addView(productView)
                }
            }
            updateInProgress = false
        }
    }

    fun removeSelected() {
        thread {
            val grv = view?.findViewById<GridLayout>(R.id.contentGridLayout)

            var deleted = false
            for (view: ProductView in grv?.children!!.iterator() as Iterator<ProductView>) {
                activity!!.runOnUiThread {
                    view.findViewById<ImageView>(R.id.productPicture).setImageURI(null)
                }
                if (view.isProductSelected) {
                    val net = Net()
                    net.token = sharedPreferences.getString("token", "")!!
                    net.server = sharedPreferences.getString("server", "")!!
                    net.language = sharedPreferences.getString("language", "en-US")!!
                    val currentGroup = sharedPreferences.getString("currentGroup", "")!!.toInt()

                    productDAO.eraseProduct(view.product.id)
                    net.deleteProduct(currentGroup, view.product.id, {response ->
                        activity!!.runOnUiThread{
                            Toast.makeText(context, response.body!!.string(), Toast.LENGTH_SHORT).show()
                        }
                    })
                    
                    deleted = true
                }
            }

            if (!deleted) {
                activity!!.runOnUiThread {
                    Toast.makeText(requireContext(), getString(R.string.nothing_to_delete), Toast.LENGTH_SHORT).show()
                }
            }
            updateContent()
        }
    }

    companion object {

        fun newInstance(date: Long):ShelfFragment = ShelfFragment().apply {
            val fragment = ShelfFragment()
            val args = Bundle()
            args.putLong("filterByExpiryDate", date)
            fragment.setArguments(args)

            return fragment
        }
    }
}