package org.foxarmy.barcodescannerforemployees.views

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.core.content.ContextCompat.startActivity
import org.foxarmy.barcodescannerforemployees.*
import org.foxarmy.barcodescannerforemployees.activities.FullscreenActivity
import org.foxarmy.barcodescannerforemployees.activities.MainActivity
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import java.io.File

class AbstractProductView : LinearLayout {
    private var productLayout: ConstraintLayout? = null
    private var productPicture: ImageView? = null
    private var productNameField: TextView? = null
    private var netWeightField: TextView? = null
    private var categoryField: TextView? = null
    private var unitField: TextView? = null
    var abstractProduct: AbstractProduct = AbstractProduct()
    var isProductSelected = false
    private var activity: Activity

    private var categoryDAO: CategoryDAO
    private var sharedPreferences: SharedPreferences

    constructor(context: Context, a: AttributeSet) : super(context, a) {
        activity = getActivity(context)!!
        val inflater: LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.abstract_product_view, this)

        sharedPreferences = getPreferences(context)

        val dbHelper = DBStorageController(context, sharedPreferences.getString("currentGroup", "database")!!)
        categoryDAO = CategoryDAO(dbHelper)
    }

    constructor(activity: Activity, context: Context, abstractProduct: AbstractProduct) : super(context) {
        this.abstractProduct = abstractProduct
        this.activity = activity
        val inflater: LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.abstract_product_view, this)

        sharedPreferences = getPreferences(context)

        val dbHelper = DBStorageController(context, sharedPreferences.getString("currentGroup", "database")!!)
        categoryDAO = CategoryDAO(dbHelper)
        update()
    }

    private fun init() {
        productLayout = findViewById(R.id.productLayout)
        productPicture = findViewById(R.id.productPicture)
        productNameField = findViewById(R.id.productNameView)
        netWeightField = findViewById(R.id.productNetWeightView)
        categoryField = findViewById(R.id.categoryView)
        unitField = findViewById(R.id.unitView)

        setUpClicks()
        unitField?.text = getUnitNameById(context, abstractProduct.unit)
    }

    fun update() {
        init()

        val thumbnailsDir = File(context.cacheDir, "thumbnails")
        thumbnailsDir.mkdirs()
        val imageUri = getImageUri(activity, File(thumbnailsDir, "${abstractProduct.imageHash}.webp"))
        productPicture!!.setImageURI(imageUri)

        productNameField!!.text = abstractProduct.name
        netWeightField!!.text = abstractProduct.netWeight.toString()
        categoryField!!.text = categoryDAO.getCategoryNameById(abstractProduct.category)
    }


    private fun setUpClicks() {
        productLayout!!.setOnLongClickListener { onAnyLongClick() }
        productNameField!!.setOnLongClickListener { onNameLongClick() }
        productPicture!!.setOnLongClickListener { onImageLongClick() }
        netWeightField!!.setOnLongClickListener { onAnyLongClick() }
        categoryField!!.setOnLongClickListener { onAnyLongClick() }
        unitField!!.setOnLongClickListener { onAnyLongClick() }

        productLayout!!.setOnClickListener { onAnyClick() }
        netWeightField!!.setOnClickListener { onAnyClick() }
        categoryField!!.setOnClickListener { onAnyClick() }
        unitField!!.setOnClickListener { onAnyClick() }
        productPicture!!.setOnClickListener { onImageLongClick() }
        productNameField!!.setOnClickListener { onNameLongClick() }
    }

    private fun onAnyClick(): Boolean {
        if (activity !is MainActivity) return false
        val mainActivity = activity as MainActivity

        if (mainActivity.selectionMode) {
            isProductSelected = !isProductSelected

            if (isProductSelected) {
                mainActivity.addSelection()
            } else {
                mainActivity.removeSelection()
            }
            activity.runOnUiThread {
                this.background = ContextCompat.getDrawable(
                    context,
                    if (isProductSelected) R.drawable.outline_selected else R.drawable.outline
                )
            }
            return true
        } else {
            return false
        }
    }

    private fun onAnyLongClick(): Boolean {
        if (activity !is MainActivity) return false
        val mainActivity = activity as MainActivity

        if (mainActivity.selectionMode) {
            isProductSelected = !isProductSelected

            if (isProductSelected) {
                mainActivity.addSelection()
            } else {
                mainActivity.removeSelection()
            }
            activity.runOnUiThread {
                this.background = ContextCompat.getDrawable(
                    context,
                    if (isProductSelected) R.drawable.outline_selected else R.drawable.outline
                )
            }
            return true
        } else {
            mainActivity.addSelection()

            isProductSelected = !isProductSelected
            activity.runOnUiThread {

                this.background = ContextCompat.getDrawable(
                    context,
                    if (isProductSelected) R.drawable.outline_selected else R.drawable.outline
                )
            }
            return true
        }
    }

    private fun onImageLongClick(): Boolean {
        if (activity !is MainActivity) {
            val fullscreenIntent = Intent(activity, FullscreenActivity::class.java)
            val extras = Bundle()
            extras.putString("imagehash", abstractProduct.imageHash)
            fullscreenIntent.putExtras(extras)
            startActivity(context, fullscreenIntent, extras)
            return true
        }
        val mainActivity = activity as MainActivity

        if (mainActivity.selectionMode) {
            return onAnyLongClick()
        } else {
            val fullscreenIntent = Intent(activity, FullscreenActivity::class.java)
            val extras = Bundle()
            extras.putString("imagehash", abstractProduct.imageHash)
            fullscreenIntent.putExtras(extras)
            startActivity(context, fullscreenIntent, extras)
        }
        return true
    }

    private fun onNameLongClick(): Boolean {
        if (activity !is MainActivity) {
            activity.runOnUiThread {
                Toast.makeText(activity, productNameField!!.text, Toast.LENGTH_SHORT).show()
            }
            return true
        }
        val mainActivity = activity as MainActivity

        if (mainActivity.selectionMode) {
            return onAnyLongClick()
        } else {
            activity.runOnUiThread {
                Toast.makeText(activity, productNameField!!.text, Toast.LENGTH_SHORT).show()
            }
        }
        return true
    }
}