package org.foxarmy.barcodescannerforemployees.views

import android.app.Activity
import android.content.Context
import android.content.SharedPreferences
import android.view.LayoutInflater
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.activities.MainActivity
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.dataclasses.Category
import org.foxarmy.barcodescannerforemployees.getPreferences

class CategoryView : LinearLayout {
    var category: Category

    val categoryName: TextView
    val amountOfProducts: TextView
    var isCategorySelected = false

    private var abstractProductDAO: AbstractProductDAO
    private var sharedPreferences: SharedPreferences

    constructor(activity: Activity, context: Context, category: Category) : super(context) {
        this.category = category

        sharedPreferences = getPreferences(context)

        val dbHelper = DBStorageController(context, sharedPreferences.getString("currentGroup", "database")!!)
        abstractProductDAO = AbstractProductDAO(dbHelper)

        val inflater: LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.category_view, this)

        updateStroke()

        categoryName = findViewById(R.id.categoryNameTextView)
        amountOfProducts = findViewById(R.id.amountOfProducts)

        categoryName.text = category.name
        amountOfProducts.text = abstractProductDAO.getAmountOfAbstractProductsInCategory(category.id).toString()

        setUpClicks()
    }

    private fun setUpClicks() {
        categoryName.setOnClickListener { onCategoryClick() }
        amountOfProducts.setOnClickListener { onCategoryClick() }

        categoryName.setOnLongClickListener { onAnyLongClick() }
        amountOfProducts.setOnLongClickListener { onAnyLongClick() }
    }

    private fun onCategoryClick(): Boolean {
        if (context !is MainActivity) return false
        val mainActivity = context as MainActivity

        if (mainActivity.selectionMode) return onAnyLongClick()
        else (context as MainActivity).filterAbstractProductsByCategory(category.id)

        return true
    }

    private fun onAnyLongClick(): Boolean {
        if (context !is MainActivity) return false
        val mainActivity = context as MainActivity

        isCategorySelected = !isCategorySelected
        updateStroke()

        if (mainActivity.selectionMode) {
            if (isCategorySelected) mainActivity.addSelection()
            else mainActivity.removeSelection()
        } else {
            mainActivity.addSelection()
        }
        return true

    }

    private fun updateStroke() {
        (context as Activity).runOnUiThread {
            this.background = ContextCompat.getDrawable(context, if (isCategorySelected) R.drawable.outline_selected else R.drawable.outline)
        }
    }
}